@@ -2,13 +2,14 @@ import io
 import json
 from collections import defaultdict
 from functools import lru_cache
-from itertools import chain
+from itertools import chain, count
 
 from django.apps import apps
 from django.conf import settings
 from django.core import serializers
 from django.core.management.color import no_style
 from django.db import DEFAULT_DB_ALIAS, connections, transaction
+from django.utils.crypto import get_random_string
 from django.utils.module_loading import import_string
 
 from feincms3_data.serializers import JSONSerializer
@@ -30,6 +31,13 @@ def _only_concrete_models(iterable):
             yield model
 
 
+def _random_values():
+    """Generate a stream of values which are unlikely to cause conflicts"""
+    prefix = get_random_string(20)
+    for i in count():
+        yield f"{prefix}-{i}"
+
+
 class InvalidSpec(Exception):
     pass
 
@@ -41,6 +49,7 @@ _valid_keys = {
     "delete_missing",
     "ignore_missing_m2m",
     "save_as_new",
+    "defer_values",
 }
 
 
@@ -108,67 +117,118 @@ def load_dump(
 
     progress(f"Loaded {len(data['objects'])} objects")
 
-    save_as_new_pk_map = defaultdict(dict)
     save_as_new_models = {
         spec["model"] for spec in data["specs"] if spec.get("save_as_new")
     }
-    ignore_missing_m2m_data = defaultdict(dict)
 
     with transaction.atomic(using=using):
         connection = connections[using]
-        models = set()
         with connection.constraint_checks_disabled():
-            for spec in data["specs"]:
-                if objs := objects[spec["model"]]:
-                    for ds in objs:
-                        if ignore_missing_m2m := spec.get("ignore_missing_m2m"):
-                            for field_name in ignore_missing_m2m:
-                                ignore_missing_m2m_data[ds][
-                                    field_name
-                                ] = ds.m2m_data.pop(field_name, [])
+            models = set()
+            _load_dump(
+                data,
+                objects,
+                progress,
+                seen_pks,
+                save_as_new_models,
+                models,
+            )
+            _finalize(
+                progress,
+                connection,
+                models,
+            )
 
-                        # _do_save changes the PK if the model is in
-                        # save_as_new_models
-                        seen_pks[ds.object._meta.label_lower].add(ds.object.pk)
-                        _do_save(
-                            ds,
-                            pk_map=save_as_new_pk_map,
-                            save_as_new_models=save_as_new_models,
-                        )
-                        seen_pks[ds.object._meta.label_lower].add(ds.object.pk)
-                        models.add(ds.object.__class__)
 
-                progress(f"Saved {len(objs)} {spec['model']} objects")
+def _load_dump(
+    data,
+    objects,
+    progress,
+    seen_pks,
+    save_as_new_models,
+    models,
+):
+    save_as_new_pk_map = defaultdict(dict)
+    ignore_missing_m2m_data = defaultdict(dict)
+    deferred_new_pks = []
+    deferred_values = []
 
-            for spec in reversed(data["specs"]):
-                if not spec.get("delete_missing"):
-                    continue
+    for spec in data["specs"]:
+        if objs := objects[spec["model"]]:
+            for ds in objs:
+                for field_name in spec.get("ignore_missing_m2m", ()):
+                    ignore_missing_m2m_data[ds][field_name] = ds.m2m_data.pop(
+                        field_name, []
+                    )
 
-                queryset = _model_queryset(spec)
-                deleted = queryset.exclude(pk__in=seen_pks[spec["model"]]).delete()
-                if deleted[0]:
-                    progress(f"Deleted {spec['model']} objects: {deleted}")
+                random_value = _random_values()
+                for field_name in spec.get("defer_values", ()):
+                    deferred_values.append(
+                        (ds, field_name, getattr(ds.object, field_name))
+                    )
+                    setattr(ds.object, field_name, next(random_value))
 
-            pks = pk_cache()
-            for ds, lists in ignore_missing_m2m_data.items():
-                for field_name, field_pks in lists.items():
-                    field = ds.object._meta.get_field(field_name)
-                    existing = pks(field.related_model)
-                    getattr(ds.object, field_name).set(set(field_pks) & existing)
+                # _do_save changes the PK if the model is in
+                # save_as_new_models
+                seen_pks[ds.object._meta.label_lower].add(ds.object.pk)
+                _do_save(
+                    ds,
+                    pk_map=save_as_new_pk_map,
+                    save_as_new_models=save_as_new_models,
+                    deferred_new_pks=deferred_new_pks,
+                )
+                seen_pks[ds.object._meta.label_lower].add(ds.object.pk)
+                models.add(ds.object.__class__)
 
-            table_names = [model._meta.db_table for model in models]
-            try:
-                connection.check_constraints(table_names=table_names)
-            except Exception as e:
-                e.args = ("Problem installing fixtures: %s" % e,)
-                raise
+        progress(f"Saved {len(objs)} {spec['model']} objects")
 
-            sequence_sql = connection.ops.sequence_reset_sql(no_style(), models)
-            if sequence_sql:
-                progress("Resetting sequences")
-                with connection.cursor() as cursor:
-                    for line in sequence_sql:
-                        cursor.execute(line)
+    _save_deferred_new_pks(deferred_new_pks)
+
+    for spec in reversed(data["specs"]):
+        if not spec.get("delete_missing"):
+            continue
+
+        queryset = _model_queryset(spec)
+        deleted = queryset.exclude(pk__in=seen_pks[spec["model"]]).delete()
+        if deleted[0]:
+            progress(f"Deleted {spec['model']} objects: {deleted}")
+
+    pks = pk_cache()
+    for ds, lists in ignore_missing_m2m_data.items():
+        for field_name, field_pks in lists.items():
+            field = ds.object._meta.get_field(field_name)
+            existing = pks(field.related_model)
+            getattr(ds.object, field_name).set(set(field_pks) & existing)
+
+    for ds, field_name, value in deferred_values:
+        setattr(ds.object, field_name, value)
+        ds.save()
+
+
+def _save_deferred_new_pks(deferred_new_pks):
+    for ds, f_name, pk_map, fk in deferred_new_pks:
+        setattr(ds.object, f_name, pk_map[fk])
+        ds.save()
+
+
+def _finalize(
+    progress,
+    connection,
+    models,
+):
+    table_names = [model._meta.db_table for model in models]
+    try:
+        connection.check_constraints(table_names=table_names)
+    except Exception as e:
+        e.args = ("Problem installing fixtures: %s" % e,)
+        raise
+
+    sequence_sql = connection.ops.sequence_reset_sql(no_style(), models)
+    if sequence_sql:
+        progress("Resetting sequences")
+        with connection.cursor() as cursor:
+            for line in sequence_sql:
+                cursor.execute(line)
 
 
 def pk_cache():
@@ -179,7 +239,10 @@ def pk_cache():
     return pks
 
 
-def _do_save(ds, *, pk_map, save_as_new_models):
+_sentinel = object()
+
+
+def _do_save(ds, *, pk_map, save_as_new_models, deferred_new_pks):
     # Map old PKs to new
     for f in ds.object._meta.get_fields():
         if (
@@ -188,7 +251,13 @@ def _do_save(ds, *, pk_map, save_as_new_models):
             and f.related_model._meta.label_lower in save_as_new_models
             and (fk := getattr(ds.object, f.column)) is not None
         ):
-            setattr(ds.object, f.name, pk_map[f.related_model][fk])
+            if (new_pk := pk_map[f.related_model].get(fk, _sentinel)) is not _sentinel:
+                setattr(ds.object, f.name, new_pk)
+            else:
+                # If foreign key isn't nullable we're toast.
+                setattr(ds.object, f.name, None)
+                # But if it is, we can defer.
+                deferred_new_pks.append((ds, f.name, pk_map[f.related_model], fk))
 
     if ds.object._meta.label_lower in save_as_new_models:
         # Do the saving
