@@ -1,4 +1,4 @@
-from django.core.management.base import BaseCommand
+from django.core.management.base import BaseCommand, CommandError
 
 from feincms3_data.data import datasets, dump_specs
 
@@ -11,10 +11,14 @@ class Command(BaseCommand):
         parser.add_argument(
             "dataset",
             help=f"Model dataset which should be dumped. {', '.join(DATASETS)}",
-            choices=DATASETS,
         )
 
     def handle(self, *args, **options):
         dataset, sep, args = options["dataset"].partition(":")
-        ds = DATASETS[dataset]
+        try:
+            ds = DATASETS[dataset]
+        except KeyError:
+            raise CommandError(
+                f"Invalid dataset {dataset}; should be one of {', '.join(DATASETS)}"
+            )
         self.stdout.write(dump_specs(ds["specs"](args), mappers=ds.get("mappers")))
